@@ -12,11 +12,14 @@ from factory import Factory
 from container import Container
 from editablelabel import EditableLabel
 from translation import _
-from util import err, dbg, enumerate_descendants
+from util import err, dbg, enumerate_descendants, make_uuid
 
 class Notebook(Container, gtk.Notebook):
     """Class implementing a gtk.Notebook container"""
     window = None
+    last_active_term = None
+    pending_on_tab_switch = None
+    pending_on_tab_switch_args = None
 
     def __init__(self, window):
         """Class initialiser"""
@@ -30,12 +33,17 @@ class Notebook(Container, gtk.Notebook):
         self.window = window
         gobject.type_register(Notebook)
         self.register_signals(Notebook)
+        self.connect('switch-page', self.deferred_on_tab_switch)
         self.configure()
 
         child = window.get_child()
         window.remove(child)
         window.add(self)
+        window_last_active_term = window.last_active_term
         self.newtab(widget=child)
+        if window_last_active_term:
+            self.set_last_active_term(window_last_active_term)
+            window.last_active_term = None
 
         self.show_all()
 
@@ -45,7 +53,7 @@ class Notebook(Container, gtk.Notebook):
         # the new order of terminals. We probably need to preserve this for
         # navigation to next/prev terminals.
         #self.connect('page-reordered', self.on_page_reordered)
-        self.set_property('homogeneous', True)
+        self.set_property('homogeneous', self.config['homogeneous_tabbar'])
         self.set_scrollable(self.config['scroll_tabbar'])
 
         if self.config['tab_position'] == 'hidden' or self.config['hide_tabbar']:
@@ -59,6 +67,12 @@ class Notebook(Container, gtk.Notebook):
             label = self.get_tab_label(self.get_nth_page(tab))
             label.update_angle()
 
+        style = gtk.RcStyle()
+        style.xthickness = 0
+        style.ythickness = 0
+        self.modify_style(style)
+        self.last_active_term = {}
+
     def create_layout(self, layout):
         """Apply layout configuration"""
         def child_compare(a, b):
@@ -112,8 +126,17 @@ class Notebook(Container, gtk.Notebook):
                     label = self.get_tab_label(page)
                     label.set_custom_label(labeltext)
             page.create_layout(children[child_key])
+
+            if  layout.get('last_active_term',  None):
+                self.last_active_term[page] = make_uuid(layout['last_active_term'][num])
             num = num + 1
 
+        if layout.has_key('active_page'):
+            # Need to do it later, or layout changes result
+            gobject.idle_add(self.set_current_page, int(layout['active_page']))
+        else:
+            self.set_current_page(0)
+
     def split_axis(self, widget, vertical=True, cwd=None, sibling=None, widgetfirst=True):
         """Split the axis of a terminal inside us"""
         dbg('called for widget: %s' % widget)
@@ -132,10 +155,16 @@ class Notebook(Container, gtk.Notebook):
         else:
             container = maker.make('hpaned')
 
+        self.get_toplevel().set_pos_by_ratio = True
+
         if not sibling:
             sibling = maker.make('terminal')
             sibling.set_cwd(cwd)
             sibling.spawn_child()
+            if widget.group and self.config['split_to_group']:
+                sibling.set_group(None, widget.group)
+        if self.config['always_split_with_profile']:
+            sibling.force_set_profile(None, widget.get_profile())
 
         self.insert_page(container, None, page_num)
         self.set_tab_reorderable(container, True)
@@ -151,7 +180,12 @@ class Notebook(Container, gtk.Notebook):
         self.set_current_page(page_num)
 
         self.show_all()
-        terminal.grab_focus()
+
+        while gtk.events_pending():
+            gtk.main_iteration_do(False)
+        self.get_toplevel().set_pos_by_ratio = False
+
+        gobject.idle_add(terminal.ensure_visible_and_focussed)
 
     def add(self, widget, metadata=None):
         """Add a widget to the container"""
@@ -195,7 +229,7 @@ class Notebook(Container, gtk.Notebook):
             children.append(self.get_nth_page(page))
         return(children)
 
-    def newtab(self, debugtab=False, widget=None, cwd=None, metadata=None):
+    def newtab(self, debugtab=False, widget=None, cwd=None, metadata=None, profile=None):
         """Add a new tab, optionally supplying a child widget"""
         dbg('making a new tab')
         maker = Factory()
@@ -206,6 +240,8 @@ class Notebook(Container, gtk.Notebook):
             if cwd:
                 widget.set_cwd(cwd)
             widget.spawn_child(debugserver=debugtab)
+        if profile and self.config['always_split_with_profile']:
+            widget.force_set_profile(None, profile)
 
         signals = {'close-term': self.wrapcloseterm,
                    'split-horiz': self.split_horiz,
@@ -214,8 +250,10 @@ class Notebook(Container, gtk.Notebook):
                    'unzoom': self.unzoom,
                    'tab-change': top_window.tab_change,
                    'group-all': top_window.group_all,
+                   'group-all-toggle': top_window.group_all_toggle,
                    'ungroup-all': top_window.ungroup_all,
                    'group-tab': top_window.group_tab,
+                   'group-tab-toggle': top_window.group_tab_toggle,
                    'ungroup-tab': top_window.ungroup_tab,
                    'move-tab': top_window.move_tab,
                    'tab-new': [top_window.tab_new, widget],
@@ -246,15 +284,29 @@ class Notebook(Container, gtk.Notebook):
 
         dbg('inserting page at position: %s' % tabpos)
         self.insert_page(widget, None, tabpos)
+
+        if maker.isinstance(widget, 'Terminal'):
+            containers, objects = ([], [widget])
+        else:
+            containers, objects = enumerate_descendants(widget)
+
+        term_widget = None
+        for term_widget in objects:
+            if maker.isinstance(term_widget, 'Terminal'):
+                self.set_last_active_term(term_widget.uuid)
+                break
+
         self.set_tab_label(widget, label)
-        self.set_tab_label_packing(widget, not self.config['scroll_tabbar'],
-                                   not self.config['scroll_tabbar'],
-                                   gtk.PACK_START)
+        gobject.idle_add(self.set_tab_label_packing,
+                             term_widget,
+                             not self.config['scroll_tabbar'],
+                             not self.config['scroll_tabbar'],
+                             gtk.PACK_START)
 
         self.set_tab_reorderable(widget, True)
         self.set_current_page(tabpos)
         self.show_all()
-        if maker.isinstance(widget, 'Terminal'):
+        if maker.isinstance(term_widget, 'Terminal'):
             widget.grab_focus()
 
     def wrapcloseterm(self, widget):
@@ -289,6 +341,7 @@ class Notebook(Container, gtk.Notebook):
 
         if maker.isinstance(child, 'Terminal'):
             dbg('Notebook::closetab: child is a single Terminal')
+            del nb.last_active_term[child]
             child.close()
             # FIXME: We only do this del and return here to avoid removing the
             # page below, which child.close() implicitly does
@@ -296,9 +349,7 @@ class Notebook(Container, gtk.Notebook):
             return
         elif maker.isinstance(child, 'Container'):
             dbg('Notebook::closetab: child is a Container')
-            dialog = self.construct_confirm_close(self.window, _('tab'))
-            result = dialog.run()
-            dialog.destroy()
+            result = self.construct_confirm_close(self.window, _('tab'))
 
             if result == gtk.RESPONSE_ACCEPT:
                 containers = None
@@ -318,9 +369,6 @@ class Notebook(Container, gtk.Notebook):
             err('Notebook::closetab: child is unknown type %s' % child)
             return
 
-        nb.remove_page(tabnum)
-        del(label)
-
     def resizeterm(self, widget, keyname):
         """Handle a keyboard event requesting a terminal resize"""
         raise NotImplementedError('resizeterm')
@@ -381,6 +429,65 @@ class Notebook(Container, gtk.Notebook):
             terms = parent.get_visible_terminals()
             terms.keys()[-1].grab_focus()
 
+    def page_num_descendant(self, widget):
+        """Find the tabnum of the tab containing a widget at any level"""
+        tabnum = self.page_num(widget)
+        dbg("widget is direct child if not equal -1 - tabnum: %d" % tabnum)
+        while tabnum == -1 and widget.get_parent():
+            widget = widget.get_parent()
+            tabnum = self.page_num(widget)
+        dbg("found tabnum containing widget: %d" % tabnum)
+        return tabnum
+
+    def set_last_active_term(self, uuid):
+        """Set the last active term for uuid"""
+        widget = self.terminator.find_terminal_by_uuid(uuid.urn)
+        if not widget:
+            err("Cannot find terminal with uuid: %s, so cannot make it active" % (uuid.urn))
+            return
+        tabnum = self.page_num_descendant(widget)
+        if tabnum == -1:
+            err("No tabnum found for terminal with uuid: %s" % (uuid.urn))
+            return
+        nth_page = self.get_nth_page(tabnum)
+        self.last_active_term[nth_page] = uuid
+
+    def clean_last_active_term(self):
+        """Clean up old entries in last_active_term"""
+        if self.terminator.doing_layout == True:
+            return
+        last_active_term = {}
+        for tabnum in xrange(0, self.get_n_pages()):
+            nth_page = self.get_nth_page(tabnum)
+            if nth_page in self.last_active_term:
+                last_active_term[nth_page] = self.last_active_term[nth_page]
+        self.last_active_term = last_active_term
+
+    def deferred_on_tab_switch(self, notebook, page,  page_num,  data=None):
+        """Prime a single idle tab switch signal, using the most recent set of params"""
+        tabs_last_active_term = self.last_active_term.get(self.get_nth_page(page_num),  None)
+        data = {'tabs_last_active_term':tabs_last_active_term}
+        
+        self.pending_on_tab_switch_args = (notebook, page,  page_num,  data)
+        if self.pending_on_tab_switch == True:
+            return
+        gobject.idle_add(self.do_deferred_on_tab_switch)
+        self.pending_on_tab_switch = True
+
+    def do_deferred_on_tab_switch(self):
+        """Perform the latest tab switch signal, and resetting the pending flag"""
+        self.on_tab_switch(*self.pending_on_tab_switch_args)
+        self.pending_on_tab_switch = False
+        self.pending_on_tab_switch_args = None
+
+    def on_tab_switch(self, notebook, page,  page_num,  data=None):
+        """Do the real work for a tab switch"""
+        tabs_last_active_term = data['tabs_last_active_term']
+        if tabs_last_active_term:
+            term = self.terminator.find_terminal_by_uuid(tabs_last_active_term.urn)
+            gobject.idle_add(term.ensure_visible_and_focussed)
+        return True
+
 class TabLabel(gtk.HBox):
     """Class implementing a label widget for Notebook tabs"""
     notebook = None
@@ -431,6 +538,9 @@ class TabLabel(gtk.HBox):
         else:
             return(None)
 
+    def edit(self):
+        self.label.edit()
+
     def update_button(self):
         """Update the state of our close button"""
         if not self.config['close_button_on_tab']:
@@ -472,6 +582,8 @@ class TabLabel(gtk.HBox):
                 self.set_orientation(gtk.ORIENTATION_VERTICAL)
             self.label.set_angle(90)
         elif position == gtk.POS_RIGHT:
+            if hasattr(self, 'set_orientation'):
+                self.set_orientation(gtk.ORIENTATION_VERTICAL)
             self.label.set_angle(270)
         else:
             if hasattr(self, 'set_orientation'):
