@@ -14,7 +14,7 @@ import gobject
 import terminatorlib.plugin as plugin
 from terminatorlib.config import Config
 from terminatorlib.translation import _
-from terminatorlib.util import get_config_dir, err, dbg
+from terminatorlib.util import get_config_dir, err, dbg, gerr
 
 (CC_COL_ENABLED, CC_COL_NAME, CC_COL_COMMAND) = range(0,3)
 
@@ -53,8 +53,8 @@ class CustomCommandsMenu(plugin.MenuItem):
                                 'command' : command
                               }
                             )
-        for cmd in noord_cmds:
-            self.cmd_list[len(self.cmd_list)] = cmd
+      for cmd in noord_cmds:
+          self.cmd_list[len(self.cmd_list)] = cmd
 
     def callback(self, menuitems, menu, terminal):
         """Add our menu items to the menu"""
@@ -295,19 +295,6 @@ class CustomCommandsMenu(plugin.MenuItem):
       dialog.show_all()
       return (dialog,enabled,name,command)
 
-    def _error(self, msg):
-      err = gtk.MessageDialog(dialog,
-                              gtk.DIALOG_MODAL,
-                              gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_CLOSE,
-                              msg
-                            )
-      err.run()
-      err.destroy()
-
-      
-
-
     def on_new(self, button, data):
       (dialog,enabled,name,command) = self._create_command_dialog()
       res = dialog.run()
@@ -338,7 +325,7 @@ class CustomCommandsMenu(plugin.MenuItem):
           if not name_exist:
             store.append((item['enabled'], item['name'], item['command']))
           else:
-            self._err(_("Name *%s* already exist") % item['name'])
+            gerr(_("Name *%s* already exist") % item['name'])
       dialog.destroy()
 
     def on_goto_top(self, button, data):
@@ -450,7 +437,7 @@ class CustomCommandsMenu(plugin.MenuItem):
                       CC_COL_COMMAND, item['command']
                       )
           else:
-            self._err(_("Name *%s* already exist") % item['name'])
+            gerr(_("Name *%s* already exist") % item['name'])
 
       dialog.destroy()
  
