@@ -116,7 +116,7 @@ class PrefsEditor:
                         'resize_right'     : _('Resize the terminal right'),
                         'move_tab_right'   : _('Move the tab right'),
                         'move_tab_left'    : _('Move the tab left'),
-                        'toggle_zoom'      : _('Maximise terminal'),
+                        'toggle_zoom'      : _('Maximize terminal'),
                         'scaled_zoom'      : _('Zoom terminal'),
                         'next_tab'         : _('Switch to the next tab'),
                         'prev_tab'         : _('Switch to the previous tab'),
@@ -148,6 +148,8 @@ class PrefsEditor:
                         'insert_number'    : _('Insert terminal number'),
                         'insert_padded'    : _('Insert padded terminal number'),
                         'edit_window_title': _('Edit window title'),
+                        'edit_terminal_title': _('Edit terminal title'),
+                        'edit_tab_title'   : _('Edit tab title'),
                         'layout_launcher'  : _('Open layout launcher window'),
                         'next_profile'     : _('Switch to next profile'),
                         'previous_profile' : _('Switch to previous profile'), 
@@ -290,6 +292,12 @@ class PrefsEditor:
         #Always split with profile
         widget = guiget('always_split_with_profile')
         widget.set_active(self.config['always_split_with_profile'])
+        # Putty paste style
+        widget = guiget('putty_paste_style')
+        widget.set_active(self.config['putty_paste_style'])
+        # Smart copy
+        widget = guiget('smart_copy')
+        widget.set_active(self.config['smart_copy'])
         #Titlebar font selector
         # Use system font
         widget = guiget('title_system_font_checkbutton')
@@ -440,13 +448,24 @@ class PrefsEditor:
         # Cursor blink
         widget = guiget('cursor_blink')
         widget.set_active(self.config['cursor_blink'])
-        # Cursor colour
+        # Cursor colour - Radio values
+        if self.config['cursor_color_fg']:
+            widget = guiget('cursor_color_foreground_radiobutton')
+        else:
+            widget = guiget('cursor_color_custom_radiobutton')
+        widget.set_active(True)
+        # Cursor colour - swatch
         widget = guiget('cursor_color')
+        widget.set_sensitive(not self.config['cursor_color_fg'])
         try:
-            widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
-        except ValueError:
-            self.config['cursor_color'] = "#FFFFFF"
-            widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+             widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+        except (ValueError, TypeError):
+            try:
+                self.config['cursor_color'] = self.config['foreground_color']
+                widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+            except ValueError:
+                self.config['cursor_color'] = "#FFFFFF"
+                widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
 
         ## Command tab
         # Login shell
@@ -722,6 +741,16 @@ class PrefsEditor:
         self.config['copy_on_selection'] = widget.get_active()
         self.config.save()
 
+    def on_putty_paste_style_toggled(self, widget):
+        """Putty paste style setting changed"""
+        self.config['putty_paste_style'] = widget.get_active()
+        self.config.save()
+
+    def on_smart_copy_toggled(self, widget):
+        """Putty paste style setting changed"""
+        self.config['smart_copy'] = widget.get_active()
+        self.config.save()
+
     def on_cursor_blink_toggled(self, widget):
         """Cursor blink setting changed"""
         self.config['cursor_blink'] = widget.get_active()
@@ -843,9 +872,12 @@ class PrefsEditor:
         self.config['scrollbar_position'] = value
         self.config.save()
 
-    def on_darken_background_scale_change_value(self, widget, scroll, value):
+    def on_darken_background_scale_change_value(self, widget, scroll, _value_not_rounded):
         """Background darkness setting changed"""
-        self.config['background_darkness'] = round(value, 2)
+        value = widget.get_value()  # This one is rounded according to the UI.
+        if value > 1.0:
+          value = 1.0
+        self.config['background_darkness'] = value
         self.config.save()
 
     def on_background_image_filechooser_file_set(self, widget):
@@ -940,6 +972,26 @@ class PrefsEditor:
         self.config['custom_command'] = widget.get_text()
         self.config.save()
 
+    def on_cursor_color_type_toggled(self, widget):
+        guiget = self.builder.get_object
+
+        customwidget = guiget('cursor_color_custom_radiobutton')
+        colorwidget = guiget('cursor_color')
+        
+        colorwidget.set_sensitive(customwidget.get_active())
+        self.config['cursor_color_fg'] = not customwidget.get_active()
+        
+        try:
+            colorwidget.set_color(gtk.gdk.color_parse(self.config['cursor_color']))
+        except (ValueError, TypeError):
+            try:
+                self.config['cursor_color'] = self.config['foreground_color']
+                colorwidget.set_color(gtk.gdk.color_parse(self.config['cursor_color']))
+            except ValueError:
+                self.config['cursor_color'] = "#FFFFFF"
+                colorwidget.set_color(gtk.gdk.color_parse(self.config['cursor_color']))
+        self.config.save()
+
     def on_cursor_color_color_set(self, widget):
         """Cursor colour changed"""
         self.config['cursor_color'] = color2hex(widget)
@@ -1002,16 +1054,18 @@ class PrefsEditor:
         self.config['title_transmit_fg_color'] = color2hex(widget)
         self.config.save()
 
-    def on_inactive_color_offset_change_value(self, widget, scroll, value):
+    def on_inactive_color_offset_change_value(self, widget, scroll, _value_not_rounded):
         """Inactive color offset setting changed"""
+        value = widget.get_value()  # This one is rounded according to the UI.
         if value > 1.0:
           value = 1.0
-        self.config['inactive_color_offset'] = round(value, 2)
+        self.config['inactive_color_offset'] = value
         self.config.save()
 
-    def on_handlesize_change_value(self, widget, scroll, value):
+    def on_handlesize_change_value(self, widget, scroll, _value_not_rounded):
         """Handle size changed"""
-        value = int(value)
+        value = widget.get_value()  # This one is rounded according to the UI.
+        value = int(value)          # Cast to int.
         if value > 5:
             value = 5
         self.config['handle_size'] = value
