@@ -76,12 +76,16 @@ class Terminal(gtk.VBox):
     }
 
     TARGET_TYPE_VTE = 8
+    TARGET_TYPE_MOZ = 9
+
+    MOUSEBUTTON_LEFT = 1
+    MOUSEBUTTON_MIDDLE = 2
+    MOUSEBUTTON_RIGHT = 3    
 
     terminator = None
     vte = None
     terminalbox = None
     scrollbar = None
-    scrollbar_position = None
     titlebar = None
     searchbar = None
 
@@ -239,19 +243,9 @@ class Terminal(gtk.VBox):
 
         terminalbox = gtk.HBox()
         self.scrollbar = gtk.VScrollbar(self.vte.get_adjustment())
-        self.scrollbar.set_no_show_all(True)
-        self.scrollbar_position = self.config['scrollbar_position']
 
-        if self.scrollbar_position not in ('hidden', 'disabled'):
-            self.scrollbar.show()
-
-        if self.scrollbar_position == 'left':
-            func = terminalbox.pack_end
-        else:
-            func = terminalbox.pack_start
-
-        func(self.vte)
-        func(self.scrollbar, False)
+        terminalbox.pack_start(self.vte, True, True, 0)
+        terminalbox.pack_start(self.scrollbar, False, True, 0)
         terminalbox.show_all()
 
         return(terminalbox)
@@ -261,7 +255,7 @@ class Terminal(gtk.VBox):
         userchars = "-A-Za-z0-9"
         passchars = "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
         hostchars = "-A-Za-z0-9"
-        pathchars = "-A-Za-z0-9_$.+!*(),;:@&=?/~#%'\""
+        pathchars = "-A-Za-z0-9_$.+!*(),;:@&=?/~#%'"
         schemes   = "(news:|telnet:|nntp:|file:/|https?:|ftps?:|webcal:)"
         user      = "[" + userchars + "]+(:[" + passchars + "]+)?"
         urlpath   = "/[" + pathchars + "]*[^]'.}>) \t\r\n,\\\"]"
@@ -335,6 +329,10 @@ class Terminal(gtk.VBox):
         self.vte.match_remove(self.matches[name])
         del(self.matches[name])
 
+    def maybe_copy_clipboard(self):
+        if self.config['copy_on_selection'] and self.vte.get_has_selection():
+            self.vte.copy_clipboard()
+
     def connect_signals(self):
         """Connect all the gtk signals and drag-n-drop mechanics"""
 
@@ -347,7 +345,7 @@ class Terminal(gtk.VBox):
 
         srcvtetargets = [("vte", gtk.TARGET_SAME_APP, self.TARGET_TYPE_VTE)]
         dsttargets = [("vte", gtk.TARGET_SAME_APP, self.TARGET_TYPE_VTE), 
-                      ('text/x-moz-url', 0, 0), 
+                      ('text/x-moz-url', 0, self.TARGET_TYPE_MOZ), 
                       ('_NETSCAPE_URL', 0, 0)]
         dsttargets = gtk.target_list_add_text_targets(dsttargets)
         dsttargets = gtk.target_list_add_uri_targets(dsttargets)
@@ -371,10 +369,8 @@ class Terminal(gtk.VBox):
         self.vte.connect('drag-data-received',
             self.on_drag_data_received, self)
 
-        # FIXME: Shouldn't this be in configure()?
-        if self.config['copy_on_selection']:
-            self.cnxids.new(self.vte, 'selection-changed', 
-                    lambda widget: self.vte.copy_clipboard())
+        self.cnxids.new(self.vte, 'selection-changed', 
+            lambda widget: self.maybe_copy_clipboard())
 
         if self.composite_support:
             self.vte.connect('composited-changed', self.reconfigure)
@@ -663,18 +659,39 @@ class Terminal(gtk.VBox):
                                                       getattr(self.fgcolor_inactive, "blue")))
         colors = self.config['palette'].split(':')
         self.palette_active = []
-        self.palette_inactive = []
         for color in colors:
             if color:
                 newcolor = gtk.gdk.color_parse(color)
-                newcolor_inactive = newcolor.copy()
-                for bit in ['red', 'green', 'blue']:
-                    setattr(newcolor_inactive, bit,
-                            getattr(newcolor_inactive, bit) * factor)
                 self.palette_active.append(newcolor)
-                self.palette_inactive.append(newcolor_inactive)
-        self.vte.set_colors(self.fgcolor_active, self.bgcolor,
-                            self.palette_active)
+        if len(colors) == 16:
+            # RGB values for indices 16..255 copied from vte source in order to dim them
+            shades = [0, 95, 135, 175, 215, 255]
+            for r in xrange(0, 6):
+                for g in xrange(0, 6):
+                    for b in xrange(0, 6):
+                        newcolor = gtk.gdk.Color(shades[r] / 255.0,
+                                                 shades[g] / 255.0,
+                                                 shades[b] / 255.0)
+                        self.palette_active.append(newcolor)
+            for y in xrange(8, 248, 10):
+                newcolor = gtk.gdk.Color(y / 255.0,
+                                         y / 255.0,
+                                         y / 255.0)
+                self.palette_active.append(newcolor)
+        self.palette_active = self.palette_active[:255]
+        self.palette_inactive = []
+        for color in self.palette_active:
+            newcolor = gtk.gdk.Color()
+            for bit in ['red', 'green', 'blue']:
+                setattr(newcolor, bit,
+                        getattr(color, bit) * factor)
+            self.palette_inactive.append(newcolor)
+        if self.terminator.last_focused_term == self:
+            self.vte.set_colors(self.fgcolor_active, self.bgcolor,
+                                self.palette_active)
+        else:
+            self.vte.set_colors(self.fgcolor_inactive, self.bgcolor,
+                                self.palette_inactive)
         self.set_cursor_color()
         if hasattr(self.vte, 'set_cursor_shape'):
             self.vte.set_cursor_shape(getattr(vte, 'CURSOR_SHAPE_' +
@@ -764,16 +781,14 @@ class Terminal(gtk.VBox):
         self.vte.set_scroll_on_keystroke(self.config['scroll_on_keystroke'])
         self.vte.set_scroll_on_output(self.config['scroll_on_output'])
 
-        if self.scrollbar_position != self.config['scrollbar_position']:
-            self.scrollbar_position = self.config['scrollbar_position']
-            if self.config['scrollbar_position'] in ['disabled', 'hidden']:
-                self.scrollbar.hide()
-            else:
-                self.scrollbar.show()
-                if self.config['scrollbar_position'] == 'left':
-                    self.reorder_child(self.scrollbar, 0)
-                elif self.config['scrollbar_position'] == 'right':
-                    self.reorder_child(self.vte, 0)
+        if self.config['scrollbar_position'] in ['disabled', 'hidden']:
+            self.scrollbar.hide()
+        else:
+            self.scrollbar.show()
+            if self.config['scrollbar_position'] == 'left':
+                self.terminalbox.reorder_child(self.scrollbar, 0)
+            elif self.config['scrollbar_position'] == 'right':
+                self.terminalbox.reorder_child(self.vte, 0)
 
         if hasattr(self.vte, 'set_alternate_screen_scroll'):
             self.vte.set_alternate_screen_scroll(
@@ -784,7 +799,7 @@ class Terminal(gtk.VBox):
 
     def set_cursor_color(self):
         """Set the cursor color appropriately"""
-        if self.config['cursor_color'] == self.config['foreground_color']:
+        if self.config['cursor_color_fg']:
             try:
                 self.vte.set_color_cursor(None) 
             except TypeError:
@@ -844,13 +859,14 @@ class Terminal(gtk.VBox):
             dbg('Terminal::on_keypress: Called on %s with no event' % widget)
             return(False)
 
-        # Workaround for IBus intefering with broadcast when using dead keys
+        # Workaround for IBus interfering with broadcast when using dead keys
         # Environment also needs IBUS_DISABLE_SNOOPER=1, or double chars appear
         # in the receivers.
-        if (event.state | gtk.gdk.MODIFIER_MASK ) ^ gtk.gdk.MODIFIER_MASK != 0:
-            dbg('Terminal::on_keypress: Ingore processed event with event.state %d' % event.state)
-            return(False)
-
+        if self.terminator.ibus_running:
+            if (event.state | gtk.gdk.MODIFIER_MASK ) ^ gtk.gdk.MODIFIER_MASK != 0:
+                dbg('Terminal::on_keypress: Ignore processed event with event.state %d' % event.state)
+                return(False)
+        
         # FIXME: Does keybindings really want to live in Terminator()?
         mapping = self.terminator.keybindings.lookup(event)
 
@@ -858,8 +874,7 @@ class Terminal(gtk.VBox):
             return(False)
 
         if mapping and mapping not in ['close_window', 
-                                       'full_screen', 
-                                       'new_tab']:
+                                       'full_screen']:
             dbg('Terminal::on_keypress: lookup found: %r' % mapping)
             # handle the case where user has re-bound copy to ctrl+<key>
             # we only copy if there is a selection otherwise let it fall through
@@ -868,6 +883,8 @@ class Terminal(gtk.VBox):
                 if self.vte.get_has_selection ():
                     getattr(self, "key_" + mapping)()
                     return(True)
+                elif not self.config['smart_copy']:
+                    return(True)
             else:
                 getattr(self, "key_" + mapping)()
                 return(True)
@@ -895,20 +912,27 @@ class Terminal(gtk.VBox):
             # Suppress double-click behavior
             return True
 
-        if event.button == 1:
+        if self.config['putty_paste_style']:
+            middle_click = [self.popup_menu, (widget, event)]
+            right_click = [self.paste_clipboard, (True, )]
+        else:
+            middle_click = [self.paste_clipboard, (True, )]
+            right_click = [self.popup_menu, (widget, event)]
+
+        if event.button == self.MOUSEBUTTON_LEFT:
             # Ctrl+leftclick on a URL should open it
             if event.state & gtk.gdk.CONTROL_MASK == gtk.gdk.CONTROL_MASK:
                 url = self.check_for_url(event)
                 if url:
                     self.open_url(url, prepare=True)
-        elif event.button == 2:
+        elif event.button == self.MOUSEBUTTON_MIDDLE:
             # middleclick should paste the clipboard
-            self.paste_clipboard(True)
+            middle_click[0](*middle_click[1])
             return(True)
-        elif event.button == 3:
+        elif event.button == self.MOUSEBUTTON_RIGHT:
             # rightclick should display a context menu if Ctrl is not pressed
             if event.state & gtk.gdk.CONTROL_MASK == 0:
-                self.popup_menu(widget, event)
+                right_click[0](*right_click[1])
                 return(True)
 
         return(False)
@@ -1045,7 +1069,7 @@ class Terminal(gtk.VBox):
         return(False)
 
     def on_drag_data_received(self, widget, drag_context, x, y, selection_data,
-            _info, _time, data):
+            info, _time, data):
         """Something has been dragged into the terminal. Handle it as either a
         URL or another terminal."""
         dbg('drag data received of type: %s' % selection_data.type)
@@ -1053,6 +1077,12 @@ class Terminal(gtk.VBox):
            gtk.targets_include_uri(drag_context.targets):
             # copy text with no modification yet to destination
             txt = selection_data.data
+
+            # https://bugs.launchpad.net/terminator/+bug/1518705
+            if info == self.TARGET_TYPE_MOZ:
+                 txt = txt.decode('utf-16').encode('utf-8')
+                 txt = txt.split('\n')[0]
+
             txt_lines = txt.split( "\r\n" )
             if txt_lines[-1] == '':
                 for line in txt_lines[:-1]:
@@ -1159,8 +1189,9 @@ class Terminal(gtk.VBox):
 
     def on_vte_focus_in(self, _widget, _event):
         """Inform other parts of the application when focus is received"""
-        self.vte.set_colors(self.fgcolor_active, self.bgcolor,
-                            self.palette_active)
+        if self.config['inactive_color_offset'] < 1.0:
+            self.vte.set_colors(self.fgcolor_active, self.bgcolor,
+                                self.palette_active)
         self.set_cursor_color()
         if not self.terminator.doing_layout:
             self.terminator.last_focused_term = self
@@ -1175,8 +1206,9 @@ class Terminal(gtk.VBox):
 
     def on_vte_focus_out(self, _widget, _event):
         """Inform other parts of the application when focus is lost"""
-        self.vte.set_colors(self.fgcolor_inactive, self.bgcolor,
-                            self.palette_inactive)
+        if self.config['inactive_color_offset'] < 1.0:
+            self.vte.set_colors(self.fgcolor_inactive, self.bgcolor,
+                                self.palette_inactive)
         self.set_cursor_color()
         self.emit('focus-out')
 
@@ -1369,6 +1401,7 @@ class Terminal(gtk.VBox):
         envv = []
         envv.append('TERM=%s' % self.config['term'])
         envv.append('COLORTERM=%s' % self.config['colorterm'])
+        envv.append('PWD=%s' % self.cwd)
         envv.append('TERMINATOR_UUID=%s' % self.uuid.urn)
         if self.terminator.dbus_name:
             envv.append('TERMINATOR_DBUS_NAME=%s' % self.terminator.dbus_name)
@@ -1766,7 +1799,10 @@ class Terminal(gtk.VBox):
         self.emit('ungroup-tab')
 
     def key_new_window(self):
-        self.terminator.new_window(self.terminator.pid_cwd(self.pid))
+        self.terminator.new_window(self.terminator.pid_cwd(self.pid), self.get_profile())
+
+    def key_new_tab(self):
+        self.get_toplevel().tab_new(self)
 
     def key_new_terminator(self):
         spawn_new_terminator(self.origcwd, ['-u'])
@@ -1820,6 +1856,20 @@ class Terminal(gtk.VBox):
         dialog.destroy()
         return
 
+    def key_edit_tab_title(self):
+        window = self.get_toplevel()
+        if not window.is_child_notebook():
+            return
+
+        notebook = window.get_children()[0]
+        n_page = notebook.get_current_page()
+        page = notebook.get_nth_page(n_page)
+        label = notebook.get_tab_label(page)
+        label.edit()
+
+    def key_edit_terminal_title(self):
+        self.titlebar.label.edit()
+
     def key_layout_launcher(self):
         LAYOUTLAUNCHER=LayoutLauncher()
 
@@ -1844,7 +1894,7 @@ class Terminal(gtk.VBox):
     def key_help(self):
         manual_index_page = manual_lookup()
         if manual_index_page:
-            self.open_url('file://%s' % (manual_index_page))
+            self.open_url(manual_index_page)
 
 # End key events
 
